/****************************************************************************
*  accelerometer.c: accelerometer API.
*
*  Although it would be perfectly OK to communicate directly with the
*  harware, this approach is more generic as it allows for f.i. on-the-fly
*  driver loading or changing. 
*  When the application uses this API it communicates through a device 
*  driver with the actual hardware.
*  Use standard C file functions like fopen, fread & fwrite to talk to it.
*
*  Author: Clemens Valens
*  License: GNU General Public License
*
*  History
*  2009.11.22  ver 1.00
*
*****************************************************************************/

/**
	@file
	High-level accelerometer driver implementation.
 */

#include <stdio.h>
#include "sceptre.h"
#include "accelerometer.h"


static FILE *p_accelerometer = NULL;


/**
* Initialize accelerometer driver.
*
* @return true on success, false otherwise.
*/
bool_t accelerometer_init(void)
{
  p_accelerometer = fopen("/accelero/1000000","r"); // Set ADC clock to 1 MHz.
  if (p_accelerometer!=NULL) return true;
  return false;
}


/**
* Read the accelerometer.
*
* @param p_dst  Address to write data to.
* @return The number of bytes read on success, -1 otherwise.
*/
int accelerometer_read(float *p_dst)
{
  if (p_accelerometer!=NULL && p_dst!=NULL)
  {
    return fread(p_dst,sizeof(float),3,p_accelerometer);
  }
  return -1;
}


/**
* Close the accelerometer driver.
*/
void accelerometer_close(void)
{
  if (p_accelerometer!=NULL)
  {
    fclose(p_accelerometer);
    p_accelerometer = NULL;
  }
}
