/*************************************************************************
Author:   Detlev Tietjen
License:  GNU General Public License

Modified by CPV for Sceptre
 *   History
 *   2009.11.05  ver 1.00    Preliminary version, first release

*************************************************************************/

/**
	@file
	Low-level BTM-112, BTM-222 Bluetooth module driver implementation.
 */

/*
BTM112
ATB? returns 0012-6f-01f533
*/

#define DBG_UART0  0

#include "sceptre.h"
#include "btm222.h"
#include <string.h>
#include <stdio.h>

const char btm222_ok_str[] = "OK";
const char btm222_error_str[] = "ERROR";
const char btm222_connect_str[] = "CONNECT";
const char btm222_disconnect_str[] = "DISCONNECT";
char btm222_peer_id[BTM222_MAX_PEER_ID+1] = "<not connected>";

btm222_device_t btm222_devices[BTM_MAX_DEVICES];
int32_t btm222_n_o_devices = 0;
bool_t btm222_connected = false;
char *p_connect = NULL;
char *p_disconnect = NULL;
char *p_peer = NULL;

bool_t btm222_connect_hook(uint32_t ch);

enum
{
  connect_hook_idle = 0,
  connect_hook_armed,
  connect_hook_waiting_for_id,
  connect_hook_collecting_id,
};

uint8_t connect_hook_mode = connect_hook_idle;
bool_t connect_hook_reconnect = false;


bool_t btm222_putc(uint8_t ch)
{
  // Send character.
  uart1_putc(ch,true);
  //printf(" %02x",ch);
  //uart0_putc(ch);
  /*char debug[10];
  sprintf(debug,"%02x",ch);
  uart0_putc(debug[0]);
  uart0_putc(debug[1]);
  uart0_putc('.');*/

  // Wait for echo or time out after 100 ms.
  uint32_t c;
  if (uart1_getc(&c,100)==false)
  {
#if DBG_UART0!=0
    uart0_putc('@');
#endif
    return false; // Time out.
  }
  if ((uint8_t)c!=ch) return false; // Echo incorrect.

  return true;
}


bool_t btm222_write(uint8_t *p_data, uint32_t data_size)
{
  bool_t result = true;
  for (int i=0; i<data_size; i++)
  {
    if (btm222_putc(p_data[i])==false) result = false; // Remember error, but continue.
  }
  return result;
}


void btm222_write_ex(uint8_t *p_data)
{
  // Send command to module. Ignore errors.
  btm222_write(p_data,strlen((char*)p_data));

  // Wait for answer or time out.
  uint32_t last_count = timer_get_count(SYSTEM_TIMER);
  while (timer_get_count(SYSTEM_TIMER)-last_count<200)
  {
    uint32_t ch;
    if (uart1_getc(&ch,10)==true) // character received?
    {
#if DBG_UART0!=0
      uart0_putc(ch);
#endif
    }
  }
}


bool_t btm222_write_command(uint8_t *p_cmd)
{
  const char *p_ok = btm222_ok_str;
  const char *p_error = btm222_error_str;

  // Send command to module. Ignore errors.
  btm222_write(p_cmd,strlen((char*)p_cmd));
  uint32_t last_count = timer_get_count(SYSTEM_TIMER);

  // Wait for answer or time out.
  while (*p_ok && *p_error)
  {
    uint32_t ch;
    if (uart1_getc(&ch,10)==true) // character received?
    {
#if DBG_UART0!=0
      uart0_putc(ch);
#endif
      if (ch==*p_ok) p_ok++;
      else p_ok = btm222_ok_str;
      if (ch==*p_error) p_error++;
      else p_error = btm222_error_str;
    }
    if (timer_get_count(SYSTEM_TIMER)-last_count>=200) return false; // Time out after 200 ms.
  }

  return *p_ok=='\0'? true : false; // Return true if the module answered "OK".
}


bool_t btm222_command(char *p_cmd, char *p_data, uint8_t data_size_max)
{
  /*char buffer[24];
  strcpy(buffer,"AT");
  strcat(buffer,p_cmd);
  if (p_data!=0 && data_size_max!=0)
  {
    // Add no more than data_size_max characters.
    strncat(buffer,p_data,data_size_max);
  }
  strcat(buffer,"\r");
  return btm222_write_command((uint8_t*)buffer);*/
  btm222_write((uint8_t*)"AT",2);
  btm222_write((uint8_t*)p_cmd,strlen(p_cmd));
  btm222_write((uint8_t*)p_data,MIN(strlen(p_data),data_size_max));
  btm222_putc('\r');
  return true;
}


uint8_t btm222_seek_devices(void)
{
  uint8_t loops;
  uint8_t i;
  char buffer[64];
  char * p;
  char * q;

  for (i=0; i<BTM_MAX_DEVICES; i++)
  { //Clear Array
    strcpy(btm222_devices[i].name,"");
    strcpy(btm222_devices[i].name,"");
  }

  btm222_n_o_devices = -1;

  if (btm222_command("F?",0,0)==false) return 0;

  p = buffer;
  *p = '\0'; // clear buffer

  loops = 30; // 30*2000 ms = 1 minute timeout
  uint32_t last_count = timer_get_count(SYSTEM_TIMER);
  while (loops && btm222_n_o_devices<8)
  {
    if (timer_get_count(SYSTEM_TIMER)-last_count>=2000)
    {
#if DBG_UART0!=0
      uart0_putc('.');
#endif
      last_count = timer_get_count(SYSTEM_TIMER);
      loops -= 1;
    }
    
    uint32_t ch;
    if (uart1_getc(&ch,10)==true) // character received?
    {
#if DBG_UART0!=0
      uart0_putc(ch);
#endif    
      if (ch>=' ')
      {
        // put into buffer
        *p++ = ch;
        *p = '\0';
      }
    
      if (ch=='\r') // End of line?
      {
        if (strstr(buffer,"Inquiry Results")) btm222_n_o_devices = 0; // start reading
        else if (strstr(buffer, "Inquiry End")) loops = 0; // end of seeking
        else if (btm222_n_o_devices>=0)
        {
          p = buffer;
          while ((*p) && (*p != '1'+btm222_n_o_devices)) p++; // seek number of device
          if (*p) // found?
          {
            for (i=0; i<3; i++) if (*p) p++;  // skip spaces
            q = btm222_devices[btm222_n_o_devices].name;
            for (i=0; i<18; i++) if (*p) *q++ = *p++;  // copy name
            *q = '\0';
            q = btm222_devices[btm222_n_o_devices].id;
            for (i=0; i<14; i++) if (*p) *q++ = *p++;    // copy id
            *q = '\0';
            btm222_n_o_devices++;
          }
        }
        p = buffer;
        *p = '\0'; // clear buffer
      }
    }
  }
  return btm222_n_o_devices;
}


void btm222_get_parameter(uint8_t param)
{
  char str[10];
  sprintf(str,"AT%c?\r",param);
  
  // Send command to module. Ignore errors.
  btm222_write((uint8_t*)str,strlen(str));

  // Wait for answer or time out.
  uint32_t last_count = timer_get_count(SYSTEM_TIMER);
  while (timer_get_count(SYSTEM_TIMER)-last_count<200)
  {
    uint32_t ch;
    if (uart1_getc(&ch,10)==true) // character received?
    {
//#if DBG_UART0!=0
      uart0_putc(ch);
//#endif
    }
  }
}


void btm222_get_all_parameters(void)
{
  btm222_get_parameter('B');
  btm222_get_parameter('D');
  btm222_get_parameter('E');
  //btm222_get_parameter('F');
  btm222_get_parameter('H');
  btm222_get_parameter('I');
  btm222_get_parameter('K');
  btm222_get_parameter('L');
  btm222_get_parameter('M');
  btm222_get_parameter('N');
  btm222_get_parameter('O');
  btm222_get_parameter('P');
  btm222_get_parameter('Q');
  btm222_get_parameter('R');
  btm222_get_parameter('Z');
}


void btm222_connect_hook_arm(bool_t reconnect)
{
  p_connect = (char *)btm222_connect_str;
  p_disconnect = (char *)btm222_disconnect_str;
  connect_hook_mode = connect_hook_armed;
  connect_hook_reconnect = reconnect;
}


bool_t btm222_connect_hook(uint32_t ch)
{
  // We are looking for messages like:
  //   CONNECT  '0014-A4-FCAC79'
  //   DISCONNECT  '0014-A4-FCAC79'
  
  //uart0_putc(ch);

  switch (connect_hook_mode)
  {
    case connect_hook_idle:
      // Nothing to do.
      break;

    case connect_hook_armed:
      // Watch out for "CONNECT" or "DISCONNECT".
      if (ch==*p_connect) p_connect++;
      else p_connect = (char *)btm222_connect_str;
      if (ch==*p_disconnect) p_disconnect++;
      else p_disconnect = (char *)btm222_disconnect_str;
      
      if (p_connect-btm222_connect_str>=strlen(btm222_connect_str))
      {
        // We saw "CONNECT", we will check later if started with "DIS" or not.
        // For now, switch to collecting the peer id.
        connect_hook_mode = connect_hook_waiting_for_id;
        p_peer = btm222_peer_id;
      }
      break;

    case connect_hook_waiting_for_id:
      // Wait for opening "'".
      if (ch=='\'') connect_hook_mode = connect_hook_collecting_id;
      break;
    
    case connect_hook_collecting_id:
      // Collect until closing "\r".
      if (ch=='\r')
      {
        if (p_disconnect-btm222_disconnect_str>=strlen(btm222_disconnect_str))
        {
          // We saw "CONNECT" but it was actually a "DISCONNECT".
          btm222_connected = false;
          // Restart waiting for a connect or not.
          if (connect_hook_reconnect==true) btm222_connect_hook_arm(true);
          else connect_hook_mode = connect_hook_idle;
        }
        else
        {
          // We saw a real "CONNECT".
          btm222_connected = true;
          // Clear UART buffers.
          uart1_flush_buffers();
          // Start watching for a "DISCONNECT".
          btm222_connect_hook_arm(connect_hook_reconnect);
          return true; // Eat this last character.
        }
      }
      else if (ch=='\'')
      {
        // Swallow trailing "'".
      }
      else if (p_peer-btm222_peer_id<=BTM222_MAX_PEER_ID)
      {
        // Add char if space allows.
        *p_peer++ = ch;
        *p_peer = '\0';
      }
      break;
  }
  
  return false; // Don't eat character.
}


bool_t btm222_is_connected(void)
{
  return btm222_connected;
}


bool_t btm222_wait_for_connect(uint32_t timeout_ms, bool_t reconnect)
{
  btm222_connect_hook_arm(reconnect);
  uart1_set_rx_hook(btm222_connect_hook);
  
  // Wait for answer or time out.
  uint32_t last_count = timer_get_count(SYSTEM_TIMER);
  while (timer_get_count(SYSTEM_TIMER)-last_count<timeout_ms)
  {
    if (btm222_is_connected()==true) break;
  }
  
  if (btm222_is_connected()==true)
  {
    //
    uart1_flush_buffers();
    return true;
  }
  return false;
}


char *btm222_get_peer_id(void)
{
  return btm222_peer_id;
}
