/*************************************************************************
Author:   Detlev Tietjen
License:  GNU General Public License 
*************************************************************************/

/**
	@file
	Low-level BTM-112, BTM-222 Bluetooth module driver interface.
 */

#ifndef __BTM222_H__
#define __BTM222_H__


#define BTM_CMD_HANDSHAKE_OFF   "ATC0\r"
#define BTM_CMD_HANDSHAKE_ON    "ATC1\r"
#define BTM_CMD_GET_LOCAL_ID    "ATB?\r"
#define BTM_CMD_ECHO_OFF        "ATE0\r"
#define BTM_CMD_ECHO_ON         "ATE1\r"
#define BTM_CMD_HIDE            "ATH0\r"
#define BTM_CMD_RECOVER         "ATH1\r"
#define BTM_CMD_STOPBIT_ONE     "ATK0\r"
#define BTM_CMD_STOPBIT_TWO     "ATK1\r"
#define BTM_CMD_4800BPS         "ATL0\r"
#define BTM_CMD_9600BPS         "ATL1\r"
#define BTM_CMD_19200BPS        "ATL2\r"
#define BTM_CMD_38400BPS        "ATL3\r"
#define BTM_CMD_PARITY_NONE     "ATM0\r"
#define BTM_CMD_PARITY_ODD      "ATM1\r"
#define BTM_CMD_PARITY_EVEN     "ATM2\r"
#define BTM_CMD_CONNECT_AUTO    "ATO0\r"
#define BTM_CMD_CONNECT_NOAUTO  "ATO1\r"
#define BTM_CMD_ROLE_MASTER     "ATR0\r"
#define BTM_CMD_ROLE_SLAVE      "ATR1\r"
#define BTM_CMD_RESET           "ATZ0\r"

#define BTM_MAX_DEVICES 8
#define BTM222_MAX_PEER_ID  16

typedef struct
{
  char name[20];
  char id[15];
} 
btm222_device_t;


extern btm222_device_t btm222_devices[BTM_MAX_DEVICES];
extern int32_t btm222_n_o_devices;


bool_t btm222_putc(uint8_t ch);
bool_t btm222_write(uint8_t *p_data, uint32_t data_size);
void btm222_write_ex(uint8_t *p_data);
bool_t btm222_write_command(uint8_t *p_cmd);
bool_t btm222_command(char *p_cmd, char *p_data, uint8_t data_size_max);

/* Seeks up to 8 bluetooth-devices. *
 * Fills the array btm222_devices[] *
 * with their names and IDs         *
 * return value is the number of    *
 * the devices (also written to     *
 * btm_n_o_devices)                 */
uint8_t btm222_seek_devices(void);

void btm222_get_parameter(uint8_t param);
void btm222_get_all_parameters(void);

void btm222_connect_hook_arm(bool_t reconnect);
bool_t btm222_is_connected(void);
bool_t btm222_wait_for_connect(uint32_t timeout_ms, bool_t reconnect);
char *btm222_get_peer_id(void);


#endif // __BTM222_H__


