/**
	@file
	User provided timer function for FatFs module.
 */

#include "sceptre.h"


/**
* This is a real time clock service to be called from     
* FatFs module. Any valid time must be returned even if   
* the system does not support a real time clock.
*         
* This is not required in read-only configuration.
*
* @return Time as an unsigned 32-bit word. 
*         bit 31:25   year from 1980 (0..127)
*         bit 24:21   month (1..12)
*         bit 20:16   day in month(1..31)
*         bit 15:11   hour (0..23)
*         bit 10: 5   minute (0..59)
*         bit  4: 0   second/2 (0..29)
*/
uint32_t get_fattime(void)
{
  uint32_t res;
  rtc_time_t rtc;

  rtc_get_time(&rtc);

  res = (((uint32_t)rtc.year - 1980) << 25)
      | ((uint32_t)rtc.month << 21)
      | ((uint32_t)rtc.day_of_month << 16)
      | (uint16_t)(rtc.hours << 11)
      | (uint16_t)(rtc.minutes << 5)
      | (uint16_t)(rtc.seconds >> 1);

  return res;
}

