/******************************************************
 *
 * SSP interface routines.
 *
 * (C) 2005 - Tom Walsh tom@openhardware.net
 *
 *
******************************************************/

/**
	@file
	SSP interface routines.
 */

#include "LPC214x.h"
#include "type.h"
#include "ssp.h"


#define SSP_PINMASK    0x000003fc
#define SSP_PINSEL     0x000002a8


/******************************************************
 *
 * ssp_init - transport media to communicate with the Flash drive.
 *
******************************************************/
void ssp_init(void)
{
  // Setup basic operation of the SSP controller.

  // Set pins for SSP operation.
  PINSEL1 = (PINSEL1 & ~SSP_PINMASK) | SSP_PINSEL;

  // Set clock rate to approx 3.93MHz (58.98MHz / 15);
  SSPCPSR = 2;

  // Just turn on master mode for now.
  // Clock is rising edge, pre-drive data bit before clock.
  // Most significant bit shifted out first.
  SSPCR0 = (SSP_DATA_SIZE(8) | SSP_FRAME_FORMAT_SPI);
  SSPCR1 = SSP_SSE;
}


static unsigned char dummyReader;

void ssp_put_byte(uint8 ch)
{
	// Spit a byte of data at the MMC.
  SSPDR = (REG16) ch;
  while (SSPSR & SSP_BSY);
  // Dummy read clears SSP BSY flag on LPC2xxx processors.
  dummyReader = (uint8) SSPDR;
}


uint8 ssp_get_byte(void)
{
	// Read one byte from the MMC card.
  SSPDR = (REG16) 0xff;
  while (SSPSR & SSP_BSY);
  return (uint8) SSPDR;
}


/***************************************************************
 *
 * ssp_send
 *
 *   Send N bytes from buf into SSP
 *
***************************************************************/
void ssp_send(const uint8 *p_data, long data_size)
{
  uint8 dummy;
  if (data_size==0) return;
  while (data_size!=0)
  {
    // As long as TNF bit is set, TxFIFO isn't full, write.
    while ((SSPSR&0x02)==0);
    SSPDR = *p_data;
    // Wait until the Busy bit is cleared.
    while ((SSPSR&0x04)==0);
    dummy = SSPDR; // Flush the RxFIFO.
    data_size--;
    p_data++;
  }
  return;
}


/***************************************************************
 *
 * ssp_read
 *
 * Reads N bytes into buffer
 *
***************************************************************/
void ssp_read(uint8 *p_data, long data_size)
{
  for (int i=0; i<data_size; i++)
  {
    SSPDR = (REG16) 0xff;
    while (SSPSR & SSP_BSY);
    p_data[i] = SSPDR;
  }
}
