/**
	@file
	Low-level device driver interface.
 */

#ifndef __DEVICE_H__
#define __DEVICE_H__


#include <reent.h>
#include <errno.h>


#ifndef STDIN_FILENO
#define STDIN_FILENO  0
#endif

#ifndef STDOUT_FILENO
#define STDOUT_FILENO  1
#endif

#ifndef STDERR_FILENO
#define STDERR_FILENO  2
#endif


// Device ID's
#define SCEPTRE_STDIN  STDIN_FILENO  /* 0 */
#define SCEPTRE_STDOUT  STDOUT_FILENO  /* 1 */
#define SCEPTRE_STDERR  STDERR_FILENO  /* 2 */
#define SCEPTRE_UART0  3
#define SCEPTRE_UART1  4
#define SCEPTRE_USB  5
#define SCEPTRE_I2C0  6
#define SCEPTRE_I2C1  7
#define SCEPTRE_SPI0  8
#define SCEPTRE_SPI1  9
#define SCEPTRE_THERMOMETER  10
#define SCEPTRE_ACCELEROMETER  11
#define SCEPTRE_BLUETOOTH  SCEPTRE_UART1
#define SCEPTRE_SDCARD  13

#define DEVICE_MAX_DESCRIPTORS  4   /* This is always max devices + 1 */


/// Device number is high byte of FILE "pointer".
#define DEVICE(D)  (D<<8)
#define DEVICE_TYPE(D)  ((D>>8)&0xff)


/// Describes what is found at driver level.
typedef const struct
{
  const char *p_name;
  // Constant to define device type: mmc, etc.
  uint16_t device_type;
  // Device methods for newlib interface.
  int (*open)(struct _reent *p_reent, const char *p_name, int flags, int mode);
  int (*close)(struct _reent *p_reent, int file);
  _ssize_t (*read)(struct _reent *p_reent, int file, void *p_dst, size_t dst_size);
  _ssize_t (*write)(struct _reent *p_reent, int file, const void *p_src, size_t src_size);
  // Init the device / software layers.
  int (*init)(void);
  // and the venerable catch-22...
  int (*ioctl)(struct _reent *p_reent, int file, int cmd, void *p_data);
} 
device_table_entry_t;


typedef const struct 
{
  const device_table_entry_t *p_item;
} 
device_table_array_t;


//typedef const device_table_array_t * device_table_list_t;

extern device_table_array_t device_table_list[DEVICE_MAX_DESCRIPTORS];


int device_find(struct _reent *r, uint8_t device_type);
void device_init(void);
int device_find_by_name(const char *p_name, int *p_len);


#endif  // __DEVICE_H__
