/**
	@file
	newlib - Implementation for fopen().
 */

#include "sceptre.h"
#include "device.h"

/**
* This function has to be implemented if you want to use fopen in your programs.
*
* All devices must start with a name and name is bracketed with '/',
* like this: "/thermo/..."
* What follows after the second '/' is passed to the device driver.
*
* @param p_reent     Pointer to a structure that will receive the return value.
* @param p_filename  Pointer to the name of the file to open.
* @param flags       Flags that will be passed to the device driver that should open the file.
* @param mode        Mode that will be passed to the device driver that should open the file.
* @return Handle to the opened file on success, -1 on failure.
*/
int _open_r(struct _reent *p_reent, const char *p_filename, int flags, int mode)
{
  int i;
  int len;
 
  //printf("_open_r(%s,%d,%d)\n",p_filename,flags,mode);
   
  // All devices must start with a name and name is bracketed with '/',
  // like this: "/thermo/..."
  // What follows after the second '/' is passed to the device driver.
  i = device_find_by_name(p_filename,&len);
  if (i>=0 && device_table_list[i].p_item->open!=0)
  {
    // Call the registered function.
    return device_table_list[i].p_item->open(p_reent,&p_filename[len+2],flags,mode);
  }

  //printf("_open_r failed\n");

  p_reent->_errno = ENODEV;
  return -1;
}
