/**
	@file
	newlib - read.
 */

#include "sceptre.h"
#include "device.h"
#include <stdio.h>


/**
* This function has to be implemented if you want to use fread in your programs.
*
* @param p_reent    Pointer to a structure that will receive the return value.
* @param fd         Handle of file to read from.
* @param p_dst      Address to write the data to.
* @param data_size  Number of bytes to read.
* @return Number of bytes written on success, -1 on failure.
*/
_ssize_t _read_r(struct _reent *p_reent, int fd, _PTR p_dst, size_t dst_size)
{
  int index;

  //printf("_read_r(0x%08x,%d,0x%08x,%d)\n",p_reent,fd,p_dst,dst_size);

  // Zero read always returns 0.
  if (dst_size==0) return dst_size;

  switch (fd)
  {
    case SCEPTRE_STDIN:
      {
        int i;
        char *p_str = (char *)p_dst;
        for (i=0; i<dst_size; i++)
        {
          uint32_t ch;
          if (uart0_getc(&ch,0)==false) break; // Will block!
          *p_str++ = ch;
        }
        return dst_size - i;
      }
      break;

    case SCEPTRE_STDOUT:
    case SCEPTRE_STDERR:
      break;
  }

  // Look through device table for this device.
  if ((index=device_find(p_reent,DEVICE_TYPE(fd)))==-1) return -1;

  // Call function if defined.
  if (device_table_list[index].p_item->read)
  {
    return device_table_list[index].p_item->read(p_reent,fd,p_dst,dst_size);
  }

  printf("*** _read_r failed\n");

  p_reent->_errno = EBADF;
  return -1;
}


