/**
	@file
	newlib - sbrk.

  Heap "management".
 */

#include "sceptre.h"
#include <reent.h>


/**
* This one is defined in the linker script and points to
* the beginning of unused RAM.
*/
extern char end[];  

static char * heap = 0;


/**
* This function has to be implemented if you want to use malloc in your programs.
*
* Grow the heap.
*
* @param p_reent  Pointer to a structure that will receive the return value.
* @param incr     The number of bytes to grow the heap with.
*/
void *_sbrk_r(struct _reent *ptr, ptrdiff_t incr)
{
  char * ret;
  //if (heap==0) heap = (char *) end;
  if (heap==0) heap = end;
  
  // Check to see if RAM is exhausted.
  if ((unsigned)(heap + incr) >= TARGET_END_OF_RAM) 
  {
    ret = (void *)-1;
  } 
  else 
  {
    // Nope, RAM is still available to malloc(),
    // note address and effectively reserve it.
    ret = heap; 
    heap += incr;
  }
  
  // Return address of newly allocated RAM.
  return (void *) ret;
}

