/**
	@file
	newlib - write.
 */

#include "sceptre.h"
#include "device.h"


/**
* This function has to be implemented if you want to use fwrite in your programs.
*
* @param p_reent    Pointer to a structure that will receive the return value.
* @param fd         Handle of file to write to.
* @param p_data     Pointer to data to write.
* @param data_size  Number of bytes to write.
* @return Number of bytes written on success, -1 on failure.
*/
_ssize_t _write_r(struct _reent *p_reent, int fd, const void *p_data, size_t data_size)
{
  int index;

  // Zero write always returns 0.
  if (data_size==0) return data_size;
  
  switch (fd)
  {
    case SCEPTRE_STDIN:
      break;

    case SCEPTRE_STDOUT:
    case SCEPTRE_STDERR:
      {
      	int i;
      	char *p_str = (char *)p_data;
        for (i=0; i<data_size; i++)
        {
          if (*p_str=='\n')
          {
            if (uart0_putc('\r')==false) break;
          }
          if (uart0_putc(*p_str++)==false) break;
        }

        return data_size - i;
      }
      break;
  }
    
  // Look thru device table for this device.
  if ((index = device_find(p_reent,DEVICE_TYPE(fd)))==-1) return -1;
  
  // Call function if defined.
  if (device_table_list[index].p_item->write) 
  {
    return device_table_list[index].p_item->write(p_reent,fd,p_data,data_size);
  }

  p_reent->_errno = EBADF;
  return -1;
}

