/**
  @file
  Main include file for Sceptre.

  Defines some useful macros and includes useful files.
  Use together with the library "sceptre.a"

  The objective is that only this file has to be included
  to use the Sceptre library. The easy way to accomplish
  this is to include everything in this file...
 */

#ifndef __SCEPTRE_H__
#define __SCEPTRE_H__


/// Only use types from stdint.h if possible!
#include <stdint.h>


#ifndef bool_t_defined
typedef enum
{
  false = 0,
  true
}
bool_t;
#define bool_t_defined  1
#endif


#ifndef NULL
#define NULL  ((void *)0)
#endif

#ifndef MIN
#define MIN(x,y)  ((x)<(y)?(x):(y))
#endif

#ifndef MAX
#define MAX(x,y)  ((x)>(y)?(x):(y))
#endif

#ifndef sizeof_array
#define sizeof_array(x)  ((sizeof(x))/(sizeof(x[0])))
#endif


#include "system/lpc214x.h"
#include "system/target.h"
#include "system/irq.h"
#include "time/timer.h"
#include "time/rtc.h"
#include "uart/uart.h"
#include "accelerometer/accelerometer.h"
#include "accelerometer/accelerometer.h"
#include "bluetooth/bluetooth.h"
//#include "filesystem/filesystem.h"

#ifndef __SKIP_FATFS__
#include "filesystem/fatfs/ff.h"
#include "filesystem/fatfs/diskio.h"
#endif // __SKIP_FATFS__

#include "newlib/device.h"
#include "thermometer/thermometer.h"
#include "usb/usb_mass_storage.h"


#endif // __SCEPTRE_H__
