/*
 *  target.h: System configuration, definitions for Fosc, Fcclk, Fcco & Fpclk.
 *
 *  Header file for Philips LPC214x Family Microprocessors
 *
 *  Copyright(C) 2006, Philips Semiconductor
 *  All rights reserved.
 *
 *  History
 *  2010.01.17  ver 1.01    CPV mods for Sceptre
 *  2005.10.01  ver 1.00    Preliminary version, first release
 *
 */

/**
	@file
	System configuration, definitions for Fosc, Fcclk, Fcco & Fpclk.
 */

#ifndef __TARGET_H__
#define __TARGET_H__


#ifdef __cplusplus
  extern "C" {
#endif


// System configuration: Fosc, Fcclk, Fcco, Fpclk must be defined.

/**
Crystal frequency, must be in range 10-25 MHz. 
*/
#define Fosc  12000000  /* Hz */

/**
System frequency, instruction clock (CCLK).
Must be a multiple of Fosc (1-32) and should be <=60 MHz.
*/
#define Fcclk  (Fosc*5)  /* Hz */

/**
CCO frequency, PLL output.
Must be a multiple of Fcclk (2,4,8 or 16) and in the range of 156-320 MHz.
*/
#define Fcco  (Fcclk*4)  /* Hz */

/** 
VPB clock frequency (PCLK).
Must be a multiple of Fcclk/4 (1,2 or 4).

lpcusb library requires that Fpclk be 60 MHz.
A lower frequency (15 MHz) would allow for lower power consumption.
*/
#define Fpclk  ((Fcclk/4)*4)  /* Hz */

/**
This is the end of the heap.
*/
#define TARGET_END_OF_RAM  0x40007fff

/**
Initialise the processor.
*/
void target_init(void);


#ifdef __cplusplus
  }
#endif


#endif // __TARGET_H
