/****************************************************************************
*  thermometer.c: thermometer API.
*
*  Although it would be perfectly OK to communicate directly with the
*  harware, this approach is more generic as it allows for f.i. on-the-fly
*  driver loading or changing. 
*  When the application uses this API it communicates through a device 
*  driver with the actual hardware.
*  Use standard C file functions like fopen, fread & fwrite to talk to it.
*
*  Author: Clemens Valens
*  License: GNU General Public License
*
*  History
*  2009.11.22  ver 1.00
*
*****************************************************************************/

/**
	@file
	High-level thermometer driver implementation.
 */

#include <stdio.h>
#include "sceptre.h"
#include "thermometer.h"


static FILE *p_thermometer = NULL;


/**
* Initialize thermometer driver.
*
* @return true on success, false otherwise.
*/
bool_t thermometer_init(void)
{
  p_thermometer = fopen("/thermo/","rw");
  if (p_thermometer!=NULL) return true;
  return false;
}


/**
* Read the thermometer.
*
* @return The temperature in degrees Celsius.
*/
float thermometer_read(void)
{
  float result = THERMOMETER_INVALID;
  if (p_thermometer!=NULL)
  {
    fread(&result,sizeof(float),1,p_thermometer);
  }
  return result;
}


/**
* Write the thermometer.
*
* @param p_data     Pointer to data to write.
* @param data_size  Number of data bytes to write.
* @return The number of bytes written.
*/
int thermometer_write(uint8_t *p_data, uint32_t data_size)
{
  if (p_thermometer!=NULL)
  {
    return fwrite(p_data,sizeof(uint8_t),data_size,p_thermometer);
  }
  return 0;
}
