/*****************************************************************************
 *   rtc.h:  Header file for Philips LPC214x Family Microprocessors
 *
 *   Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2005.10.01  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/

// modified by Martin Thomas

/**
	@file
	Definitions for the real time clock (RTC) driver.
 */

#ifndef __RTC_H__
#define __RTC_H__

typedef struct
{
  uint32_t seconds; //!< [0,59]
  uint32_t minutes; //!< [0,59]
  uint32_t hours;  //!< [0,23]
  uint32_t day_of_month;  //!< [1,31]
  uint32_t day_of_week;  //!< [0,6]
  uint32_t day_of_year;  //!< [1,365]
  uint32_t month;  //!< [1,12]
  uint32_t year;  //!< [0,4095]
}
rtc_time_t;

#define RTC_TIME_BASE  0xe0024020
#define RTC_ALARM_BASE  0xe0024060


#define IMSEC		0x00000001
#define IMMIN		0x00000002
#define IMHOUR		0x00000004
#define IMDOM		0x00000008
#define IMDOW		0x00000010
#define IMDOY		0x00000020
#define IMMON		0x00000040
#define IMYEAR		0x00000080

#define AMRSEC		0x00000001  /* Alarm mask for Seconds */
#define AMRMIN		0x00000002  /* Alarm mask for Minutes */
#define AMRHOUR		0x00000004  /* Alarm mask for Hours */
#define AMRDOM		0x00000008  /* Alarm mask for Day of Month */
#define AMRDOW		0x00000010  /* Alarm mask for Day of Week */
#define AMRDOY		0x00000020  /* Alarm mask for Day of Year */
#define AMRMON		0x00000040  /* Alarm mask for Month */
#define AMRYEAR		0x00000080  /* Alarm mask for Year */

//#include "target.h"
#define PREINT_RTC  ((Fpclk/32768)-1)
#define PREFRAC_RTC (Fpclk-((PREINT_RTC+1)*32768))

#if 0
#define PREINT_RTC	0x000001C8  /* Prescaler value, integer portion, PCLK = 15Mhz */
#define PREFRAC_RTC	0x000061C0  /* Prescaler value, fraction portion, PCLK = 15Mhz */
#endif

#define ILR_RTCCIF	0x01
#define ILR_RTCALF	0x02

#define CCR_CLKEN	0x01
#define CCR_CTCRST	0x02
#define CCR_CLKSRC	0x10

extern void rtc_init(void);
extern void rtc_start(void);
extern void rtc_stop(void);
extern void rtc_clear_tick_counter(void);
extern void rtc_set_time(const rtc_time_t *p_time);
extern void rtc_get_time(rtc_time_t *p_time);
extern void rtc_set_alarm(const rtc_time_t *p_time);
extern void rtc_set_alarm_mask(uint32_t mask);

extern volatile uint32_t rtc_alarm_active;


#endif // __RTC_H__
/*****************************************************************************
**                            End Of File
******************************************************************************/
