/*****************************************************************************
 *   timer.h:  Header file for Philips LPC214x Family Microprocessors
 *
 *   Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2009.11.06  ver 2.00    mods by CPV
 *   2005.10.01  ver 1.00    Preliminary version, first Release
 *
******************************************************************************/

/**
	@file
	Definitions for timer0, timer1 and the system timer.

  The system timer is supposed to run at 1 kHz.

  A 100 Hz counter is available too.
 */

#ifndef __TIMER_H__
#define __TIMER_H__


#define TIMER_ZERO  0
#define TIMER_ONE  1
#define SYSTEM_TIMER  TIMER_ONE


void timer_wait_ms(uint32_t delay_ms);

typedef void (*timer_10ms_hook_t)(void);
// If a hook is registered, it will be called every 10 ms.
void timer_set_10ms_hook(timer_10ms_hook_t f);

bool_t timer_init(uint8_t timer_num, uint32_t interval);
void timer_enable(uint8_t timer_num);
void timer_disable(uint8_t timer_num);
void timer_reset(uint8_t timer_num);

uint32_t timer_get_count(uint8_t timer_num);
void timer_set_count(uint8_t timer_num, uint32_t val);
uint32_t timer_get_system_count(void);
void timer_set_system_count(uint32_t val);
uint32_t timer_get_10ms_counter(void);

volatile uint32_t timer0_counter;
volatile uint32_t timer1_counter;


#endif // __TIMER_H__
/*****************************************************************************
**                            End Of File
******************************************************************************/
