/*****************************************************************************
*   uart.h:  Header file for Philips LPC214x Family Microprocessors
*
*   Copyright(C) 2006, Philips Semiconductor
*   All rights reserved.
*
*   History
*   2010.01.08  ver 2.01    CPV - added uart_rx_hook
*   2009.12.14  ver 2.00    CPV - Added UART1
*   2005.10.01  ver 1.00    Prelimnary version, first Release
*
******************************************************************************/

/**
	@file
	Definitions for UART0 & UART1.
 */

#ifndef __UART_H__
#define __UART_H__


#define IER_RBR   0x01
#define IER_THRE  0x02
#define IER_RLS   0x04

#define IIR_PEND  0x01
#define IIR_RLS   0x03
#define IIR_RDA   0x02
#define IIR_CTI   0x06
#define IIR_THRE  0x01

#define LSR_RDR   0x01
#define LSR_OE    0x02
#define LSR_PE    0x04
#define LSR_FE    0x08
#define LSR_BI    0x10
#define LSR_THRE  0x20
#define LSR_TEMT  0x40
#define LSR_RXFE  0x80

/// If a hook returns true the character will be removed from stream.
typedef bool_t (*uart_rx_hook_t)(uint32_t ch);


bool_t uart0_init(uint32_t baudrate, uint8_t data_bits, char parity, uint8_t stop_bits);
uint32_t uart0_send(const uint8_t *p_data, uint32_t data_size, bool_t cr_to_crlf);
bool_t uart0_putc(uint32_t ch);
bool_t uart0_getc(uint32_t *p_ch, uint32_t timeout_in_ms);
void uart0_flush_buffers(void);
void uart0_set_rx_hook(uart_rx_hook_t f);


bool_t uart1_init(uint32_t baudrate, uint8_t data_bits, char parity, uint8_t stop_bits);
uint32_t uart1_send(const uint8_t *p_data, uint32_t data_size, bool_t cr_to_crlf);
bool_t uart1_putc(uint32_t ch, bool_t right_now);
bool_t uart1_getc(uint32_t *p_ch, uint32_t timeout_in_ms);
void uart1_flush_buffers(void);
void uart1_set_rx_hook(uart_rx_hook_t f);


#endif // __UART_H__
